using RaftModLoader;
using UnityEngine;
using HMLLibrary;
using HarmonyLib;
using System.Runtime.CompilerServices;
using System.Reflection;

public class PlayerSpeedChanger : Mod
{
    public static PlayerSpeedChanger instance;
    Harmony harmony;

    // 移動速度のデフォルト値
    public const float DefaultNormalSpeed = 3f;
    public const float DefaultSprintSpeed = 6f;
    public const float DefaultSwimSpeed = 2f;
    public const float DefaultJumpSpeed = 8f;

    // 移動速度の倍率
    public static float normalSpeedMultiplier = 1f;
    public static float sprintSpeedMultiplier = 1f;
    public static float swimSpeedMultiplier = 2f;
    public static float jumpSpeedMultiplier = 1f;

    // 落下ダメージ
    public static bool fallDamageDisabled = false;


    public void Start()
    {
        instance = this;
        harmony = new Harmony("com.sora.PlayerSpeedChanger");
        harmony.PatchAll();

        Debug.Log("Mod PlayerSpeedChanger has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmony.Id);
        Debug.Log("Mod PlayerSpeedChanger has been unloaded!");
    }

    [ConsoleCommand(name: "changeWalkSpeed", docs: "This command changes the player's walking speed.")]
    public static string ChangeWalkSpeedCommand(string[] args)
    {
        if (args.Length == 0)
            return "Too few arguments";
        if (args.Length > 1)
            return "Too many arguments";

        float newSpeedMultiplier;
        if (float.TryParse(args[0], out newSpeedMultiplier))
        {
            normalSpeedMultiplier = newSpeedMultiplier;
            ExtraSettingsAPI_SetSliderValue("WalkingSpeed", normalSpeedMultiplier);
            return $"Walking speed has been changed to {newSpeedMultiplier} times.";
        }
        else
        {
            return "Invalid argument: Please provide a valid number.";
        }
    }

    [ConsoleCommand(name: "changeSprintSpeed", docs: "This command changes the player's sprint speed.")]
    public static string ChangeSprintSpeedCommand(string[] args)
    {
        if (args.Length == 0)
            return "Too few arguments";
        if (args.Length > 1)
            return "Too many arguments";

        float newSpeedMultiplier;
        if (float.TryParse(args[0], out newSpeedMultiplier))
        {
            sprintSpeedMultiplier = newSpeedMultiplier;
            ExtraSettingsAPI_SetSliderValue("SprintSpeed", sprintSpeedMultiplier);
            return $"Sprint speed has been changed to {newSpeedMultiplier} times.";
        }
        else
        {
            return "Invalid argument: Please provide a valid number.";
        }
    }

    [ConsoleCommand(name: "changeSwimSpeed", docs: "This command changes the player's swimming speed.")]
    public static string ChangeSwimSpeedCommand(string[] args)
    {
        if (args.Length == 0)
            return "Too few arguments";
        if (args.Length > 1)
            return "Too many arguments";

        float newSpeedMultiplier;
        if (float.TryParse(args[0], out newSpeedMultiplier))
        {
            swimSpeedMultiplier = newSpeedMultiplier;
            ExtraSettingsAPI_SetSliderValue("SwimmingSpeed", swimSpeedMultiplier);
            return $"Swimming speed has been changed to {newSpeedMultiplier} times.";
        }
        else
        {
            return "Invalid argument: Please provide a valid number.";
        }
    }

    [ConsoleCommand(name: "changeJumpSpeed", docs: "This command changes the player's jump speed.")]
    public static string ChangeJumpSpeedCommand(string[] args)
    {
        if (args.Length == 0)
            return "Too few arguments";
        if (args.Length > 1)
            return "Too many arguments";

        float newSpeedMultiplier;
        if (float.TryParse(args[0], out newSpeedMultiplier))
        {
            jumpSpeedMultiplier = newSpeedMultiplier;
            ExtraSettingsAPI_SetSliderValue("JumpSpeed", jumpSpeedMultiplier);
            return $"Jump speed has been changed to {newSpeedMultiplier} times.";
        }
        else
        {
            return "Invalid argument: Please provide a valid number.";
        }
    }

    [ConsoleCommand(name: "showAllSpeeds", docs: "This command shows all speeds.")]
    public static string ShowAllSpeedsCommand(string[] args)
    {
        if (args.Length > 0)
            return "Too many arguments";

        string ret =
            $"walking speed: x{normalSpeedMultiplier}\n" +
            $"sprint speed: x{sprintSpeedMultiplier}\n" +
            $"swimming speed: x{swimSpeedMultiplier}\n" +
            $"jump speed: x{jumpSpeedMultiplier}\n" +
            $"fall damage: {(fallDamageDisabled ? "disabled" : "enabled")}";

        return ret;
    }

    [ConsoleCommand(name: "resetAllSpeeds", docs: "This command resets all speeds.")]
    public static string ResetAllSpeedsCommand(string[] args)
    {
        if (args.Length > 0)
            return "Too many arguments";

        ResetSettingValues();
        return "All speed settings have been reset to default values.";
    }

    [ConsoleCommand(name: "enableFallDamage", docs: "This command enables fall damage.")]
    public static string EnableFallDamageCommand(string[] args)
    {
        if (args.Length > 0)
            return "Too many arguments";

        fallDamageDisabled = false;
        ExtraSettingsAPI_SetCheckboxState("FallDamageDisabled", fallDamageDisabled);
        return "Fall damage enabled.";
    }

    [ConsoleCommand(name: "disableFallDamage", docs: "This command disables fall damage.")]
    public static string DisableFallDamageCommand(string[] args)
    {
        if (args.Length > 0)
            return "Too many arguments";

        fallDamageDisabled = true;
        ExtraSettingsAPI_SetCheckboxState("FallDamageDisabled", fallDamageDisabled);
        return "Fall damage disabled.";
    }

    [HarmonyPatch(typeof(PersonController), "GroundControll")]
    class Patch_GroundControll
    {
        static void Prefix(PersonController __instance)
        {
            __instance.normalSpeed = DefaultNormalSpeed * normalSpeedMultiplier;
            __instance.sprintSpeed = DefaultSprintSpeed * sprintSpeedMultiplier;
            __instance.jumpSpeed = DefaultJumpSpeed * jumpSpeedMultiplier;
            //Debug.Log("normalSpeed changed: " + DefaultNormalSpeed + " -> " + __instance.normalSpeed);
            //Debug.Log("sprintSpeed changed: " + DefaultSprintSpeed + " -> " + __instance.sprintSpeed);
            //Debug.Log("jumpSpeed changed: " + DefaultJumpSpeed + " -> " + __instance.jumpSpeed);
        }

        static void Postfix(PersonController __instance)
        {
            __instance.normalSpeed = DefaultNormalSpeed;
            __instance.sprintSpeed = DefaultSprintSpeed;
            __instance.jumpSpeed = DefaultJumpSpeed;
        }
    }

    [HarmonyPatch(typeof(PersonController), "WaterControll")]
    class Patch_WaterControll
    {
        static void Prefix(PersonController __instance)
        {
            __instance.swimSpeed = DefaultSwimSpeed * swimSpeedMultiplier;
            //Debug.Log("swimSpeed changed: " + DefaultSwimSpeed + " -> " + __instance.swimSpeed);
        }
        
        static void Postfix(PersonController __instance)
        {
            __instance.swimSpeed = DefaultSwimSpeed;
        }
    }

    [HarmonyPatch(typeof(PersonController), "HandleFalling")]
    class Patch_HandleFalling
    {
        static bool Prefix()
        {
            if (fallDamageDisabled)
                return false;
            else
                return true;
        }
    }

    public void ExtraSettingsAPI_Load()
    {
        Debug.Log("Loaded existing setting values.");
        LoadSettingValues();
    }

    public void ExtraSettingsAPI_SettingsClose()
    {
        Debug.Log("Settings menu closed. update settings values.");
        LoadSettingValues();
    }

    public void ExtraSettingsAPI_ButtonPress(string SettingName)
    {
        if (SettingName == "Reset")
        {
            ResetSettingValues();
            Debug.Log("All player speed Reset.");
        }
    }

    private void LoadSettingValues()
    {
        normalSpeedMultiplier = ExtraSettingsAPI_GetSliderValue("WalkingSpeed");
        sprintSpeedMultiplier = ExtraSettingsAPI_GetSliderValue("SprintSpeed");
        swimSpeedMultiplier = ExtraSettingsAPI_GetSliderValue("SwimmingSpeed");
        jumpSpeedMultiplier = ExtraSettingsAPI_GetSliderValue("JumpSpeed");
        fallDamageDisabled = ExtraSettingsAPI_GetCheckboxState("FallDamageDisabled");

        Debug.Log($"walking speed: x{normalSpeedMultiplier}");
        Debug.Log($"sprint speed: x{sprintSpeedMultiplier}");
        Debug.Log($"swimming speed: x{swimSpeedMultiplier}");
        Debug.Log($"jump speed: x{jumpSpeedMultiplier}");
        Debug.Log($"FallDamageDisabled: {fallDamageDisabled}");
    }

    private static void ResetSettingValues()
    {
        normalSpeedMultiplier = 1f;
        sprintSpeedMultiplier = 1f;
        swimSpeedMultiplier = 1f;
        jumpSpeedMultiplier = 1f;

        ExtraSettingsAPI_SetSliderValue("WalkingSpeed", normalSpeedMultiplier);
        ExtraSettingsAPI_SetSliderValue("SprintSpeed", sprintSpeedMultiplier);
        ExtraSettingsAPI_SetSliderValue("SwimmingSpeed", swimSpeedMultiplier);
        ExtraSettingsAPI_SetSliderValue("JumpSpeed", jumpSpeedMultiplier);

        fallDamageDisabled = false;
        ExtraSettingsAPI_SetCheckboxState("FallDamageDisabled", fallDamageDisabled);
    }

    //| Because of some issues with how things are
    //| handled, it's highly recommended, but not
    //| required, to include the MethodImpl attribute
    //| on these methods

    // Use to get the current value from a Slider type setting
    // This method returns the value of the slider rounded according to the mod's setting configuration
    [MethodImpl(MethodImplOptions.NoInlining)]
    public static float ExtraSettingsAPI_GetSliderValue(string SettingName) => 0;

    // Use to set the value of a Slider type setting
    [MethodImpl(MethodImplOptions.NoInlining)]
    public static void ExtraSettingsAPI_SetSliderValue(string SettingName, float value) { }

    // Use to get the current state of a Checkbox type setting
    [MethodImpl(MethodImplOptions.NoInlining)]
    public static bool ExtraSettingsAPI_GetCheckboxState(string SettingName) => false;

    // Use to set the current state of a Checkbox type setting
    [MethodImpl(MethodImplOptions.NoInlining)]
    public static void ExtraSettingsAPI_SetCheckboxState(string SettingName, bool value) { }
}
